@@ -1,16 +1,16 @@
+from __future__ import annotations
+
 import os
 import platform
 import re
+import shutil
 import socket
 import stat
 import subprocess
 import sys
 import tempfile
 
-try:
-    from setuptools.extern.packaging.version import Version
-except ImportError:
-    from distutils.version import StrictVersion as Version
+from packaging.version import Version
 
 tempprefix = 'hg-hghave-'
 
@@ -45,6 +45,11 @@ def _bytes2sys(p):
     return p.decode('utf-8')
 
 
+def has_cmd(cmd: str) -> bool:
+    """return True if there is an executable `cmd` in the PATH"""
+    return shutil.which(cmd, os.X_OK) is not None
+
+
 def check(name, desc):
     """Registers a check function for a feature."""
 
@@ -147,6 +152,8 @@ def matchoutput(cmd, regexp: bytes, ignorestatus: bool = False):
     is matched by the supplied regular expression.
     """
     env = environ()
+    # make sure that strict-rhg mode does not interfer with hghave call
+    env['RHG_ON_UNSUPPORTED'] = 'fallback'
 
     r = re.compile(regexp)
     p = subprocess.Popen(
@@ -437,13 +444,7 @@ def has_pygit2():
 # https://github.com/git-lfs/lfs-test-server
 @check("lfs-test-server", "git-lfs test server")
 def has_lfsserver():
-    exe = 'lfs-test-server'
-    if has_windows():
-        exe = 'lfs-test-server.exe'
-    return any(
-        os.access(os.path.join(path, exe), os.X_OK)
-        for path in os.environ["PATH"].split(os.pathsep)
-    )
+    return has_cmd('lfs-test-server')
 
 
 @checkvers("git", "git client (with ext::sh support) version >= %s", ('1.9',))
@@ -802,14 +803,12 @@ def has_curses():
 
 @check("tic", "terminfo compiler")
 def has_tic():
-    return matchoutput('test -x "`which tic`"', br'')
+    return has_cmd('tic')
 
 
 @check("xz", "xz compression utility")
 def has_xz():
-    # When Windows invokes a subprocess in shell mode, it uses `cmd.exe`, which
-    # only knows `where`, not `which`.  So invoke MSYS shell explicitly.
-    return matchoutput("sh -c 'test -x \"`which xz`\"'", b'')
+    return has_cmd('xz')
 
 
 @check("msys", "Windows with MSYS")
@@ -926,7 +925,7 @@ def has_python3exe():
     py = 'python3'
     if os.name == 'nt':
         py = 'py -3'
-    return matchoutput('%s -V' % py, br'^Python 3.(6|7|8|9|10|11)')
+    return matchoutput('%s -V' % py, br'^Python 3.(6|7|8|9|1\d)')
 
 
 @check("pure", "running with pure Python code")
@@ -962,13 +961,10 @@ def unzip_understands_symlinks():
 
 @check("zstd", "zstd Python module available")
 def has_zstd():
-    try:
-        import mercurial.zstd
-
-        del mercurial.zstd
-        return True
-    except ImportError:
-        return False
+    return matchoutput(
+        'hg debuginstall',
+        br'checking available compression engines.*zstd',
+    )
 
 
 @check("devfull", "/dev/full special file")
@@ -1048,7 +1044,7 @@ def has_extraextensions():
 
 @check('dirstate-v2', 'using the v2 format of .hg/dirstate')
 def has_dirstate_v2():
-    # Keep this logic in sync with `newreporequirements()` in `mercurial/localrepo.py`
+    # Keep this logic in sync with `new_repo_requirements()` in `mercurial/repo/creation.py`
     return matchoutput(
         'hg config format.use-dirstate-v2', b'(?i)1|yes|true|on|always'
     )
@@ -1087,7 +1083,9 @@ def has_emacs():
     # Our emacs lisp uses `with-eval-after-load` which is new in emacs
     # 24.4, so we allow emacs 24.4, 24.5, and 25+ (24.5 was the last
     # 24 release)
-    return matchoutput('emacs --version', b'GNU Emacs 2(4.4|4.5|5|6|7|8|9)')
+    return matchoutput(
+        'emacs --version', br'GNU Emacs (3\d|2(4.4|4.5|5|6|7|8|9))'
+    )
 
 
 @check(
@@ -1159,3 +1157,13 @@ def has_bash():
 @check("bigendian", "big-endian CPU")
 def has_bigendian():
     return sys.byteorder == 'big'
+
+
+@check("python-local-coerce", "Python use local coercion")
+def python_coercion():
+    return matchoutput(
+        "unset LC_ALL; unset LANG; LC_CTYPE= ;"
+        " $PYTHON -c \"import os; print(os.environ.get('LC_CTYPE',"
+        " '__NO_COERCES__'))\"",
+        b'^C.UTF-8$',
+    )
